Attribute VB_Name = "RndData"
Option Explicit

' -----------------------------------------------------------
' Build a random generated data string.
' Written by Kenneth Ives   kenaso@home.com
' -----------------------------------------------------------

' ---------------------------------------------------
' API function for obtaining the number of
' milliseconds since the last reboot.
' ---------------------------------------------------
  Private Declare Function GetTickCount Lib "kernel32" () As Long
  
' ---------------------------------------------------
' API function for obtaining volume information
' ---------------------------------------------------
  Private Declare Function GetVolumeInformation Lib "kernel32" _
          Alias "GetVolumeInformationA" (ByVal lpRootPathName As String, _
          ByVal lpVolumeNameBuffer As String, ByVal nVolumeNameSize As Long, _
          lpVolumeSerialNumber As Long, lpMaximumComponentLength As Long, _
          lpFileSystemFlags As Long, ByVal lpFileSystemNameBuffer As String, _
          ByVal nFileSystemNameSize As Long) As Long

' ---------------------------------------------------
' This ENumWindows call is one of the most useful.
' ---------------------------------------------------
  Private Declare Function EnumWindows Lib "user32" _
          (ByVal lpfn As Long, ByVal lParam As Long) As Boolean
  
  Private Declare Function EnumChildWindows Lib "user32" _
          (ByVal hWndParent As Long, ByVal lpEnumFunc As Long, _
           ByVal lParam As Long) As Long
  
' ---------------------------------------------------
' Define local variables
' ---------------------------------------------------
  Private lHwndCount As Long
  Private lParentHwnd() As Long
  
Private Function EnumChildWindowsProc(ByVal wHandle As Long, lParam As Long) As Long
  
' -----------------------------------------------------------
' Written by Kenneth Ives   kenaso@home.com
'
' Gather all of the active sub window handle values.
' -----------------------------------------------------------

' -----------------------------------------------------------
' Add the child window handle values to the counter
' -----------------------------------------------------------
  lHwndCount = lHwndCount + wHandle
  EnumChildWindowsProc = True

End Function

Private Function EnumWindowsProc(ByVal wHandle As Long, lParam As Long) As Long
    
' -----------------------------------------------------------
' Written by Kenneth Ives   kenaso@home.com
'
' Gather all of the active window handle values.
' -----------------------------------------------------------

' -----------------------------------------------------------
' Add the parent window handle values to the array
' -----------------------------------------------------------
  ReDim Preserve lParentHwnd(UBound(lParentHwnd) + 1)
  lParentHwnd(UBound(lParentHwnd)) = wHandle
  EnumWindowsProc = True

End Function

Public Function Build_Random_Data(lDataLength As Long, _
                Optional bAllChars As Boolean = True) As String

' -------------------------------------------------------------
' Written by Kenneth Ives   kenaso@home.com
'
' Build a random data string.  Keep it within reason so as not
' to clog your memory processes.  The maximum value I have ever
' used was 32768 (1024 * 32).
'
' Parameters:
'    lDataLength = Length of the data string
'
'    bAllChars   = True  (use 0 to 255)
'                  False (use 32 to 126) Printable characters
'
' -------------------------------------------------------------

' -------------------------------------------------------------
' Define local variables
' -------------------------------------------------------------
  Dim iChar As Integer
  Dim sChar As String
  Dim sTmp1 As String
  Dim sTmp2 As String
  Dim lByteCnt As Long

' -------------------------------------------------------------
' Define local variables
' -------------------------------------------------------------
  lByteCnt = 0
  sTmp1 = ""
  sTmp2 = ""
  
' ---------------------------------------------------------------
' If length of the desired data string equal zero then leave
' ---------------------------------------------------------------
  If lDataLength = 0 Then GoTo Normal_Exit
  
' ---------------------------------------------------------------
' Create a new seed for the random generator.
' The seed will look similar to these values:
'
'           11981049.3104323
'            9796065.52249338
' ---------------------------------------------------------------
  Seed_Random_Generator

' -------------------------------------------------------------
' Build a string of randomly generated characters using 1k
' chunks.  The output string can be longer if desired.
' -------------------------------------------------------------
  Do
      ' get an integer value
      If bAllChars Then
          ' Use all ASCII decimal values
          ' 0 to 255 (hex(00) to hex(FF))
          iChar = Int(Rnd * 255)
      Else
          ' Use only printable character values
          ' 32 to 126 (hex(20) to hex(7E))
          iChar = Int(Rnd * 126)    ' generate integer value 0 to 126
          
          ' Test the value.  If less than 32 then
          ' add 32
          If iChar < 32 Then iChar = iChar + 32

      End If
                    
      sChar = Chr(iChar)         ' convert to ASCII character
      sTmp1 = sTmp1 & sChar      ' Append to temp holding string
      lByteCnt = lByteCnt + 1    ' increment character count
      
      ' Build the output string in 1k chunks.
      ' If the length of the temp holding string
      ' is divisible by 1024 evenly, then append
      ' it to output string and empty it.
      If Len(sTmp1) Mod 1024 = 0 Then
          sTmp2 = sTmp2 & sTmp1    ' append to the new string
          sTmp1 = ""               ' clear the temp holding string
      End If
      
  Loop Until lByteCnt >= lDataLength
  
' -------------------------------------------------------------
' Append whatever is left in the temp holding string to the
' output string.  Will only append if the byte count is
' greater than zero.  Otherwise sTmp1 will be empty.
' -------------------------------------------------------------
  sTmp2 = sTmp2 & sTmp1
  
      
Normal_Exit:
' -------------------------------------------------------------
' Return the completed string
' -------------------------------------------------------------
  Build_Random_Data = sTmp2
  
End Function

Public Sub Seed_Random_Generator()

' -----------------------------------------------------------
' Written by Kenneth Ives   kenaso@home.com
'
' Seed the random generator using a unique seed.  Can be
' called by itself whenever you want to seed the random
' number generator for Visual Basic 5 and above (32-bit).
' -----------------------------------------------------------

' -----------------------------------------------------------
' Define variables
' -----------------------------------------------------------
  Dim k As Long
  Dim lParam As Long
  Dim lRetVal As Long
  Dim lTickCnt As Long
  Dim lVolSn As Long
  
  Dim dRndSeed As Double  ' 64-bit(8-byte) floating-point number
  Dim Pi As Double
  
' -----------------------------------------------------------
' Empty all variables for security purposes
' -----------------------------------------------------------
  Erase lParentHwnd             ' Remove all the data
  ReDim lParentHwnd(0) As Long  ' so length is not known
  lHwndCount = 0                ' reset other variables to zero
  lTickCnt = 0
  lVolSn = 0
  dRndSeed = 0
  
' -----------------------------------------------------------
' Get numeric value of all active windows
' -----------------------------------------------------------
  lRetVal = EnumWindows(AddressOf EnumWindowsProc, lParam)
    
' -----------------------------------------------------------
' Accumulate the total values of the active window handles
' -----------------------------------------------------------
  For k = 1 To UBound(lParentHwnd)
      lHwndCount = lHwndCount + lParentHwnd(k)
  Next
    
' -----------------------------------------------------------
' Get numeric value of all active child windows
' -----------------------------------------------------------
  For k = 1 To UBound(lParentHwnd)
      lRetVal = EnumChildWindows(lParentHwnd(k), AddressOf EnumChildWindowsProc, 0)
  Next

' -----------------------------------------------------------
' Calculate Pi to 14 decimal places which should be
' 3.14159265358979
' -----------------------------------------------------------
  Pi = Atn(1) * 4
  
' -----------------------------------------------------------
' Get volume serial number of drive C:
' Assumes there will always be a drive C:
' -----------------------------------------------------------
  lRetVal = GetVolumeInformation("C:\", "", 0, lVolSn, 0, 0, "", 0)
  
' -----------------------------------------------------------
' get number of milliseconds since the last reboot
' -----------------------------------------------------------
  lTickCnt = GetTickCount

' -----------------------------------------------------------
' Initialize the random generator using the system date
' and time stamp as the seed.  Randomize returns a data
' type that stores single-precision floating-point variable
' as a 32-bit(4-byte) floating-point number.
' -----------------------------------------------------------
  Randomize Timer
  
' -----------------------------------------------------------
' Create a new seed value based on the following:
'
'  1.  Number of milliseconds since the last reboot
'  2.  The value of the current date/time stamp
'  3.  The value of Pi to 14 decimal places
'  4.  Accumulated total of all active windows & sub windows
'  5.  Volume serial number of drive C:
'  6.  Square root of running total thus far
'  7.  Random generated value from Pi
'  8.  Random generated value from the system timestamp
'  9.  Random generated value from step one (Tick count)
' 10.  Random generated value from the value of one
' 11.  Random generated value from the value in step 4
' 12.  Random generated value from the value in step 5
' -----------------------------------------------------------
  dRndSeed = (lTickCnt + Timer + Pi + lHwndCount + lVolSn) + _
             Sqr(lTickCnt + Timer + Pi + lHwndCount + lVolSn) + _
             (Rnd * Pi) + (Rnd * Timer) + (Rnd * lTickCnt) + _
             (Rnd * 1) + (Rnd * lHwndCount) + (Rnd * lVolSn)
  
' -----------------------------------------------------------
' Re-Initialize the random generator using the new seed.
' Randomize returns a data type that stores single-precision
' floating-point variable as a 32-bit (4-byte)
' floating-point number.
'
' The calculated value used will look similar to this:
'
'           11981049.3104323
'            9796065.52249338
' -----------------------------------------------------------
  Randomize dRndSeed

' -----------------------------------------------------------
' Empty all variables for security purposes
' -----------------------------------------------------------
  Erase lParentHwnd             ' Remove all the data
  ReDim lParentHwnd(0) As Long  ' so length is not known
  lHwndCount = 0                ' reset other variables to zero
  lTickCnt = 0
  lVolSn = 0
  dRndSeed = 0
  
End Sub
Sub Main()

' for testing, rename this procedure to Main()
' and look in the debug immediate window for
' the results
  Dim sTmp As String
  
  sTmp = Build_Random_Data(25)
  
  Debug.Print Len(sTmp)
  Debug.Print sTmp
  
  End
  
End Sub
